/* 
   sitecopy, remote web site copy manager.
   Copyright (C) 1998, Joe Orton <joe@orton.demon.co.uk>.
                                                                     
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
  
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>

#include "common.h"

extern int quiet;

enum debug_t debug_level = debug_none;

#ifdef NEEDBASENAME
#include <unistd.h>

/* Handy basename implementation for the unpriviliged.
 * I haven't tested this, but it looks about right - JO */

char *basename( const char *filename ) {
    char *last;
    last = (char *) strrchr( filename, '/' );
    if( last == NULL ) {
	return (char *) filename;
    } else {
	return (char *) (last + 1);
    }
}
#endif

char *dirname( const char *filename ) {
    char *last, *ret;
    last = (char *) strrchr( filename, '/' );
    if( last == NULL ) {
	ret = malloc( 1 );
	*ret = '\0';
	return ret;	
    } else {
	ret = malloc( last - filename + 1 );
	strncpy( ret, filename + 1, last - filename + 1 );
	return ret;
    }
}

void debug( enum debug_t debug_min, char *template, ...) {
    va_list params;
    if( debug_level < debug_min ) return;
    fflush( stdout );
    va_start( params, template );
    vfprintf( stderr, template, params );
    va_end( params );
}

/* Base64 specification from RFC 1521 */

const char b64_alphabet[65] = { 
    "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
    "abcdefghijklmnopqrstuvwxyz"
    "0123456789+/=" };
    
char *base64( const char *text ) {
    /* The tricky thing about this is doing the padding at the end,
     * doing the bit manipulation requires a bit of concentration only */
    char *buffer, *point;
    int inlen, outlen;
    
    /* Use 'buffer' to store the output. Work out how big it should be...
     * This must be a multiple of 4 bytes */

    inlen = strlen( text );
    outlen = (inlen*4)/3;
    if( (inlen % 3) > 0 ) /* got to pad */
	outlen += 4 - (inlen % 3);
    
    buffer = malloc( outlen + 1 ); /* +1 for the \0 */
    
    /* now do the main stage of conversion, 3 bytes at a time,
     * leave the trailing bytes (if there are any) for later */

    for( point=buffer; inlen>=3; inlen-=3, text+=3 ) {
	*(point++) = b64_alphabet[ (*text)>>2 ]; 
	*(point++) = b64_alphabet[ ((*text)<<4 & 0x30) | (*(text+1))>>4 ]; 
	*(point++) = b64_alphabet[ ((*(text+1))<<2 & 0x3c) | (*(text+2))>>6 ];
	*(point++) = b64_alphabet[ (*(text+2)) & 0x3f ];
    }

    /* Now deal with the trailing bytes */
    if( inlen ) {
	/* We always have one trailing byte */
	*(point++) = b64_alphabet[ (*text)>>2 ];
	*(point++) = b64_alphabet[ ((*text)<<4 & 0x30) |
				 (inlen==2?(*(text+1))>>4:0) ]; 
	*(point++) = (inlen==1?'=':b64_alphabet[ (*(text+1))<<2 & 0x3c ] );
	*(point++) = '=';
    }
		      
    *point = '\0';

    return buffer;
}

#ifdef BASE64_TEST

int main( int argc, char **argv ) {
    char *out;
    if( argc < 2 ) exit(-1);
    out = base64( argv[1] );
    printf( "Plain: [%s], Base64: [%s]\n", argv[1], out );
    return 0;
}

#endif /* BASE64_TEST */
