/* 
   sitecopy, remote web site copy manager.
   Copyright (C) 1998, Joe Orton <joe@orton.demon.co.uk>
                                                                     
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
  
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* Protocol driver */

#ifndef PROTO_H
#define PROTO_H

#include <sys/types.h>

#define PROTO_OK 0
/* Misc error */
#define PROTO_ERROR -1
/* Hostname lookup failed */
#define PROTO_LOOKUP -2
/* Local hostname could not be found */
#define PROTO_LOCALHOST -3
/* Could not connect */
#define PROTO_CONNECT -4
/* Could not find local file */
#define PROTO_FILE -5

/* 
#define PROTO_NAME( prot ) \
prot==proto_ftp?"ftp":prot==proto_http?"http":"Unknown"
*/

enum protocol_t {
    proto_ftp, /* standard FTP server */
    proto_http, /* funky HTTP-DAV server */
};

struct proto_driver {
    /* To connect to the server */
    int (*connect) ( const char *hostname, const int port );
    /* Supply the user-specified login name and password */
    int (*login) ( const char *username, const char *password );
    /* Close the connection */
    int (*close) ( void );
    /* Perform the file operations - these should return one of 
     * the PROTO_ codes */
    int (*file_move) ( const char *from, const char *to );
    int (*file_upload) ( const char *local, const char *remote );
    int (*file_delete) ( const char *filename );
    int (*file_chmod) ( const char *filename, const mode_t mode );
    /* Perform the directory operations */
    int (*dir_create) ( const char *dirname );
    int (*dir_remove) ( const char *dirname );
    char **last_error;
    char *service_name; /* The name of the port which this protocol
				 * will use by default */
    char *protocol_name; /* The user-visible name for this protocol */
};

#endif /* PROTO_H */
