/* 
   socket handling routines
   Copyright (C) 1998, Joe Orton <joe@orton.demon.co.uk>, except where
   otherwise indicated.
                                                                     
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
  
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <sys/socket.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>
#include <stdio.h>

#include "socket.h"
#include "common.h"

/* Send the given line down the socket with CRLF appended. */
void send_line( const int sock, const char *line ) {
    char *buffer;
    buffer = malloc( strlen(line) + strlen("\r\n") + 1 );
    strcpy( buffer, line );
    /* Add \r\n on the end - Unix servers will ignore the \r, 
     * Windows ones require \r\n */
    strcat( buffer, "\r\n" );
    write(sock, buffer, strlen(buffer));
    free( buffer );
}

void send_data( const int sock, const char *data ) {
    write( sock, data, strlen( data ) );
}

/* This is from from Eric Raymond's fetchmail (SockRead() in socket.c)
 * since I wouldn't have a clue how to do it properly.
 * This function is Copyright (C) Eric Raymond.
 */
int read_line( const int sock, char *buffer, int len) {
    char *newline, *bp = buffer;
    int n;
    
    if (--len < 1)
	return(-1);
    do {
	/*
	 * The reason for these gymnastics is that we want two things:
	 * (1) to read \n-terminated lines,
	 * (2) to return the true length of data read, even if the
	 *     data coming in has embedded NULS.
	 */
	if ((n = recv(sock, bp, len, MSG_PEEK)) <= 0)
	    return(-1);
	if ((newline = memchr(bp, '\n', n)) != NULL)
	    n = newline - bp + 1;
	if ((n = read(sock, bp, n)) == -1)
	    return(-1);
	bp += n;
	len -= n;
    } while (!newline && len);
    *bp = '\0';
    return bp - buffer;
}

/* Dump the given filename down the given socket.
 * Returns non-zero value if successful */
int send_file( const int sock, const char *filename ) {
    char buffer[BUFSIZ];
    int len, wrlen, fp;
    if( (fp = open( filename, O_RDONLY )) == -1 ) {
	perror("open"); 
	return 0;
    }
    printf( "Sending %s ", filename );

    /* dump from file to socket */
    do {
	len = read( fp, buffer, BUFSIZ );
	if( len < 0 ) perror("read");
	if( len > 0 ) wrlen = write( sock, buffer, len );
	putc( '.', stdout );
	fflush( stdout ); /* Make the dots appear straight away */
    } while( len > 0 );

    close( fp );
    printf(" sent.\n");
    return 0;
}

int host_lookup( const char *hostname, struct in_addr *addr ) {
    struct hostent *hp;
    if( ( hp = gethostbyname(hostname) ) == NULL )
	return -1;
    memcpy( addr, hp->h_addr, hp->h_length );
    return 0;
}

int socket_connect( const struct in_addr addr, const int portnum ) {
    struct sockaddr_in sa;
    int sock;
    /* Look up the host name */
    /* Create the socket */
    if( ( sock = socket(AF_INET, SOCK_STREAM, 0) ) < 0)
	return -1;
    /* Connect the socket */
    sa.sin_family = AF_INET;
    sa.sin_port = htons(portnum); /* host -> net byte orders */
    sa.sin_addr = addr;
    if( connect(sock, (struct sockaddr *) &sa, sizeof(struct sockaddr_in)) < 0 )
	return -1;
    /* Success - return the socket */
    return sock;
}

void socket_close( const int sock ) {
    close( sock );
}

/* Yeah, I'm lazy.
 * This returns HOST ORDER port. */
int get_tcp_port( const char *name ) {
    struct servent *ent;
    ent = getservbyname( name, "tcp" );
    if( ent == NULL ) {
	return 0;
    } else {
	return ntohs( ent->s_port );
    }
}
